<?php
/**
 * @file
 * totem_common.inflect.inc
 */

/**
 * Class to handle plural/singular-ization and of strings. Acknowledgements
 * to work posted at.
 * @see http://kuwamoto.org/2007/12/17/improved-pluralizing-in-php-actionscript-and-ror
 */
class Inflect {

  protected static $plural = array(
    '/(quiz)$/i' => "$1zes",
    '/^(ox)$/i' => "$1en",
    '/([m|l])ouse$/i' => "$1ice",
    '/(matr|vert|ind)ix|ex$/i' => "$1ices",
    '/(x|ch|ss|sh)$/i' => "$1es",
    '/([^aeiouy]|qu)y$/i' => "$1ies",
    '/(hive)$/i' => "$1s",
    '/(?:([^f])fe|([lr])f)$/i' => "$1$2ves",
    '/(shea|lea|loa|thie)f$/i' => "$1ves",
    '/sis$/i' => "ses",
    '/([ti])um$/i' => "$1a",
    '/(tomat|potat|ech|her|vet)o$/i' => "$1oes",
    '/(bu)s$/i' => "$1ses",
    '/(alias)$/i' => "$1es",
    '/(octop)us$/i' => "$1i",
    '/(ax|test)is$/i' => "$1es",
    '/(us)$/i' => "$1es",
    '/s$/i' => "s",
    '/$/' => "s",
  );

  protected static $singular = array(
    '/(quiz)zes$/i' => "$1",
    '/(matr)ices$/i' => "$1ix",
    '/(vert|ind)ices$/i' => "$1ex",
    '/^(ox)en$/i' => "$1",
    '/(alias)es$/i' => "$1",
    '/(octop|vir)i$/i' => "$1us",
    '/(cris|ax|test)es$/i' => "$1is",
    '/(shoe)s$/i' => "$1",
    '/(o)es$/i' => "$1",
    '/(bus)es$/i' => "$1",
    '/([m|l])ice$/i' => "$1ouse",
    '/(x|ch|ss|sh)es$/i' => "$1",
    '/(m)ovies$/i' => "$1ovie",
    '/(s)eries$/i' => "$1eries",
    '/([^aeiouy]|qu)ies$/i' => "$1y",
    '/([lr])ves$/i' => "$1f",
    '/(tive)s$/i' => "$1",
    '/(hive)s$/i' => "$1",
    '/(li|wi|kni)ves$/i' => "$1fe",
    '/(shea|loa|lea|thie)ves$/i' => "$1f",
    '/(^analy)ses$/i' => "$1sis",
    '/((a)naly|(b)a|(d)iagno|(p)arenthe|(p)rogno|(s)ynop|(t)he)ses$/i' => "$1$2sis",
    '/([ti])a$/i' => "$1um",
    '/(n)ews$/i' => "$1ews",
    '/(h|bl)ouses$/i' => "$1ouse",
    '/(corpse)s$/i' => "$1",
    '/(us)es$/i' => "$1",
    '/s$/i' => "",
  );

  protected static $irregular = array(
    'move' => 'moves',
    'foot' => 'feet',
    'goose' => 'geese',
    'sex' => 'sexes',
    'child' => 'children',
    'man' => 'men',
    'tooth' => 'teeth',
    'person' => 'people',
    'media' => 'media',
  );

  protected static $uncountable = array(
    'sheep',
    'fish',
    'deer',
    'series',
    'species',
    'money',
    'rice',
    'information',
    'equipment',
    'media',
  );

  protected static $forbid_translation = array(
    'community',
    'communities',
    'event',
    'events',
    'media',
    'media-collection',
    'media-collections',
    'page',
    'pages',
    'topic',
    'topics',
    'user',
    'members',
    'recent',
    'orphan',
    'orphans',
  );

  /**
   * Function to allow modules to apply translation to final matched string.
   */
  protected static function finalize(&$result) {

    // Admin may attempt to translate machine-names, or their plurals, of
    // entity types defined in the system. (e.g. in an attempt to change path
    // aliases or something via stringoverrides). For now, we have to
    // explicitly disallow these strings; by not doing so all kinds of request
    // routing issues will arise and "unexplainable" 403s and 404s will be
    // thrown...basically because by changing these particular path strings,
    // no dynamic hook invocation is possible in totem_common_node_community,
    // in type-based blocks, etc.
    //
    // Also note that this class is called exclusively from
    // _totem_common_node_types...so we have to hardcode the forbid list here
    // so an infinite loop isn't triggered.
    if (!in_array($result, self::$forbid_translation)) {
      return t($result);
    }

    return $result;
  }
  /**
   * Function to pluralize a string.
   */
  public static function pluralize($string) {

    // Save some time in the case that singular and plural are the same.
    if (in_array(strtolower($string), self::$uncountable)) {
      return self::finalize($string);
    }

    // Check for irregular singular forms.
    foreach (self::$irregular as $pattern => $result) {
      $pattern = '/' . $pattern . '$/i';
      if (preg_match($pattern, $string)) {
        $result = preg_replace($pattern, $result, $string);
        return self::finalize($result);
      }
    }

    // Check for matches using regular expressions.
    foreach (self::$plural as $pattern => $result) {
      if (preg_match($pattern, $string)) {
        $result = preg_replace($pattern, $result, $string);
        return self::finalize($result);
      }
    }

    return self::finalize($string);
  }
  /**
   * Function to singularize a string.
   */
  public static function singularize($string) {

    // Save some time in the case that singular and plural are the same.
    if (in_array(strtolower($string), self::$uncountable)) {
      return self::finalize($string);
    }

    // Check for irregular plural forms.
    foreach (self::$irregular as $result => $pattern) {
      $pattern = '/' . $pattern . '$/i';
      if (preg_match($pattern, $string)) {
        $result = preg_replace($pattern, $result, $string);
        return self::finalize($result);
      }
    }

    // Check for matches using regular expressions.
    foreach (self::$singular as $pattern => $result) {
      if (preg_match($pattern, $string)) {
        $result = preg_replace($pattern, $result, $string);
        return self::finalize($result);
      }
    }

    return self::finalize($string);
  }
}
